#!/usr/bin/env python2
#Importe
import cv2 #OpenCV (bildverarbeitung)
import numpy as np #numppy (erforderlich fuer OpenCV)
from time import sleep
import threading
import RPi.GPIO as GPIO
import sys


#Variablen  ####################################################################
running = True
debug = True

#Motor
ausrichtung = [0.005 , 0.005] #[MotorL, MotorR]
V = 40      # Geschwindichkeit

#Lautsprecher
toene = []    #array mit den erkannten toenen : [False, False, False, False, True, False, False, False]
# Frequenzen der Noten C3 bis C7 in Hz (nicht alle genutzt)
noten = [131, 147, 165, 175, 196, 220, 247, 262, 294, 330, 349, 392, 440, 494, 523, 587, 659, 698, 784, 880, 988, 1047, 1175, 1319, 1397, 1568, 1760, 1976, 2093]
beepDelay = 0 #delay , der die Lautsprecher Frequenz bestimmt

#camera
blocksize = 25 # Breite/Hoehe von note-block
'''thresh1 =   #Kamerabild Numpy array 640x480'''

#--------------------------------Pin Deffinitionen
#Motor R
DIRR = 20
STEPR = 21
#Motor L
DIRL = 19
STEPL =26
#Lautsprecher
PIN_TON=22

#Funktionen ####################################################################

def noteerkennen():
    """Liest mit Hilfe von black() die zu spielenden Noten vom Blatt ab und schreibt sie in eine Liste.
    In dieser steht True an der stelle wo eine Note gespielt werden soll. """
    notenbild=[]
    for i in range(640/blocksize):
        if(i < ((640/blocksize)/2 - 2) or ( i > ((640/blocksize)/2))):
            notenbild.append(black((400/blocksize),i))
    return notenbild

def noten_Spielen(toene): # errechnet die delayzeit # unnoetig
    """Nimmt eine Liste von noteerkennen() entgegen rechnet diese in Frequenzen um."""
    for i in  range(len(toene)):
        if toene[i]:
            beepDelay = 1000000.0/noten[i] #halbe periode, periode =1/frequenz
            beepDelay=beepDelay/1000000.0 #umwandeln in microseconds
            return beepDelay
    beepDelay = 0
    return beepDelay

#swapped x,y
def black(y,x):
    """Sucht nach schwarze Pixel um y,x (25 Pixel)
    Returnt True wenn 15 oder mehr schwarze Pixel erkannt werden"""
    bpixels = 0
    for i in range(5):
        for j in range(5):
            if(thresh1.item((y*blocksize) + (i-2) ,x*blocksize + (j-2) + (blocksize/2)) == 0):
                bpixels += 1
    if(bpixels >= 15):
        return True
    else:
        return False

def lenken():
    """fragt die mittleren 3 Bildbloecke der Kamera ab und errechnet daraus die durschschnittliche position von dem meisten Schwarz ( also der spur ).
    Ist diese in der Mitte errechnet lenken() = 0.5 ( 50% ) und laesst beide motoren gleich schnell fahren, indem es die Frequenzen der beiden Motoren zurueck gibt.
    Weicht die Spur von den 0.5 ab wird der eine der beiden Motoren langsamer und der andere schneller um entgegen zu lenken.
    Vor dem ausgeben der Motorfrequenzen der den die durch die globale Variable V #(fuer geschwindichkeit) dividiert, um das Tempo an zu passen."""
    bild=[]
    for i in range(3*blocksize):
        bild.append(thresh1.item(blocksize,11*blocksize+i))
    n = 0
    durchschnitt = 0
    for i in range(len(bild)):
        if bild[i] != 0:
            durchschnitt += i
            n+= 1
    if n == 0:
        print("Spur nicht gefunden")
        return [0,0]
    durchschnitt /= n * float(len(bild))
    return [ durchschnitt /V , (1-durchschnitt)/V]


def cleanup_GPIO() :
  GPIO.cleanup()
  print("Motoren abgeschaltet")

#Klassen #######################################################################
class moFred(threading.Thread):
    """Threading Klasse, die einen Moror mit (Pololu A4988) Treiber steuert. Diesem Motor wird eine iD, eine Drehrichtung ( 1 oder 0 ) so wie ein STEP und ein DIR Pin zugewiesen.
    Sobald der Thread gestartet wird fragt dieser die iD'te stelle von ausrichtung[] ab und fraehrt in dieser Frequenz in die durch drehrichtung bestimmte Richtung"""
        def __init__(self, iD, STEP, DIR, drehrichtung):
                threading.Thread.__init__(self)
                self.iD = iD
                self.STEP = STEP
                self.DIR = DIR
                self.drehrichtung = drehrichtung
        def run(self):
                GPIO.setup(self.DIR, GPIO.OUT)
                GPIO.setup(self.STEP, GPIO.OUT)
                GPIO.output(self.DIR, self.drehrichtung)
                while running:
                    GPIO.output(self.STEP, GPIO.HIGH)
                    sleep(ausrichtung[self.iD])
                    GPIO.output(self.STEP, GPIO.LOW)
                    sleep(ausrichtung[self.iD])
                else:
                    cleanup_GPIO() #Bug

class noFred(threading.Thread):
    """Threading Klasse, die Lautsprecher/Piezos steuert. Dem Lautsprecher wird eine iD und ein Pin zugewiesen.
    Sobald der Therad gestartet wird Fragt dieser die Frequenz der globalen Variable beepDelay ab und spielt sie.
    ist beepDelay = 0 wird nichts gespielt"""
        def __init__(self, iD, PIN_TON):
                threading.Thread.__init__(self)
                self.iD = iD
                self.PIN_TON = PIN_TON
        def run(self):
                GPIO.setup(self.PIN_TON, GPIO.OUT)
                while running:
                        if beepDelay != 0:
                                GPIO.output(self.PIN_TON, GPIO.HIGH)
                                sleep(beepDelay)
                                GPIO.output(self.PIN_TON, GPIO.LOW)
                                sleep(beepDelay)
                        else:
                                sleep(0.005) #Bug notenverzoegerung?



#main ##########################################################################
raw_input("Start ?")

GPIO.setmode(GPIO.BCM) #init GPIO
#starte motoren
tMotorL = moFred(0, STEPL, DIRL, 1)
tMotorR = moFred(1, STEPR, DIRL, 1)
tMotorL.start()
tMotorR.start()

#starte Lautsprecher
tLautsprecher = noFred(5, PIN_TON)
tLautsprecher.start()

#starte camera
cap = cv2.VideoCapture(0)

#mainloop
while running:
    try:
        # ------- video ----------
        ret, frame = cap.read() # Macht ein Bild
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY) # Konvertiert Bild zu schwarz/weiss
        ret,thresh1 = cv2.threshold(gray,80,255,cv2.THRESH_BINARY) # Erzeugt binaeres Bild
        #--------------------------

        ausrichtung = lenken()
#        print(ausrichtung) #debug

        toene = noteerkennen()
        beepDelay = noten_Spielen(toene)


        sleep(0.05)

        if debug:   # debug Funktionen #########################################
            # Veranschaulicht was Die Kamera sieht
            '''
            string = '|'
            string2 = '-'
            for i in xrange(480/blocksize):
                for j in xrange(640/blocksize):
                    if(black(i,j)):
                        string += "   "
                        string2 += "---"
                    else:
                        string += "xxx"
                        string2 += "---"
                    string += '|'
                    string2 += '-'
                print(string2)
                print(string)
            #   print(string)
                string = '|'
                string2 = '-'
            print
            print
            '''
            print(ausrichtung[0] - ausrichtung[1])
#            print(ausrichtung)
#            print(toene, "\t" ,beepDelay )
#            print("beepDelay :  ",beepDelay)
#            print("ausrichtung :  ", ausrichtung)
#
#            print( "Running = " , running)
    except KeyboardInterrupt: #beendet das Programm bei STRG-C (SIGINT)
        running = False
        sys.exit()
